/*
 * Gioco.java
 *
 * Created on 4 dicembre 2007, 18.23
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package babaoglu;

import java.util.*;
import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import murlen.util.fscript.FSException;
import murlen.util.fscript.FScript;

/**
 * La classe Gioco è la classe principale, che gestisce tutte le altre classi e il
 * motore stesso del gioco.<br>
 * Contiene 2 Threads, uno che gestisce i comandi del giocatore, e un altro che si
 * occupa di mandare avanti a intervalli regolari il gioco intero.
 * @author dario
 */
public class Gioco implements Runnable, GiocoInterfaccia {

    /**
     * Creates a new instance of Gioco
     * @param nomeGiocatore è il nome da assegnare al Giocatore
     * @param dove contiene l'indirizzo della cartella, i cui files verranno tutti aperti per
     * trarne informazioni per il gioco.<br>
     * I files <b>.script</b> contengono gli script da aggiungere al motore FScript.<br>
     * I files <b>.babaoglu</b> contengono le descrizioni degli oggetti, delle locazioni, 
     * delle pietre miliari e dei personaggi.
     * @param d è l'implementazione dell'interfaccia Destinazione
     */
    public Gioco(File dove, BabaIO d/*, String nomeGioc, String descrGioc, String locGioc*/) {

        // Carica il gioco
        if (!dove.isDirectory()) {
            System.out.println("Non hai passato una directory corretta");
            return;



        } else {
            try {

                // Imposto la BabaIO
                dest = d;

                // Crea l'interprete del gioco
                interprete = new FScript();
                interprete.registerExtension(new Estensione(this));

                this.listaNPC = new LinkedHashMap<String, NPC>();
                this.locazioni = new LinkedHashMap<String, Locazione>();
                this.oggetti = new LinkedHashMap<String, Oggetto>();
                this.pietreMiliari = new LinkedHashMap<String, PietraMiliare>();

                for (File f : dove.listFiles()) {
                    if (f.isFile()) {
                        String[] risultato = f.getName().split("[.]");

                        if (risultato[risultato.length - 1].equals("script")) {
                            try {
                                FileReader esso = new FileReader(f);
                                interprete.load(esso);
                            } catch (IOException ex) {
                                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        } else if (risultato[risultato.length - 1].equals("baba")) {
                            leggiBaba(f);
                        } else if (risultato[risultato.length - 1].equals("loc")) {
                            leggiLoc(f);
                        }
                    }
                }

                // Avvio l'interprete
                interprete.run();


                // Creo il Player
                ArrayList<String> param = new ArrayList<String>();
                param.add("getNomePlayer");
                String name = (String) interprete.callScriptFunction("inizio_func", param);

                param.clear();
                param.add("getDescrPlayer");
                String descrPlayer = (String) interprete.callScriptFunction("inizio_func", param);

                giocatore = new Player(name, descrPlayer);

                param.clear();
                param.add("getLocIniziale");
                giocatore.setLocazione((String) interprete.callScriptFunction("inizio_func", param));

//                for (String pietre : pietreMiliari.keySet()) {
//                    System.out.println(pietre);
//                }
                
//                PietraMiliare gatto = pietreMiliari.get("inizio");
//                String fece = gatto.getEvento();
//                System.out.println(fece);
                
                dest.impostaTitolo(pietreMiliari.get("inizio").getEvento());


                //Crea il thread che muove il gioco
                tempo = new Thread(new Tempo(d, this));

                finito = false;

            // Ora qualcuno dovrà chiamare la funzione run!
            } catch (IOException ex) {
                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
            } catch (FSException ex) {
                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
            }

        // Ora qualcuno dovrà chiamare la funzione run!

        }



    }

    private void leggiLoc(File f) {

        if (f.isFile()) {
            BufferedReader lettore = null;
            Blocco attuale = null;
            try {
                lettore = new BufferedReader(new FileReader(f));

                String riga;
                // Stati:
                // 0 = non ho trovato niente
                // 1 = ho trovato un blocco
                int stato = 0;

                while ((riga = lettore.readLine()) != null) {

                    // Sono nello stato 0
                    if (stato == 0) {

                        // Controllo se la riga contiene caratteri
                        // Metto > 3 perché [a] sono già tre caratteri, per esempio
                        if (riga.length() > 3) {
                            // Controllo se i caratteri sono [ e ] all'inizio e alla fine
                            if (riga.startsWith("[") && riga.endsWith("]")) {

                                // Se ho già trovato qualcosa, lo scrivo
                                if (attuale != null) {
//                                    System.out.println("Devo scrivere la loc appena trovata");
                                    locazioni.put(attuale.id, new Locazione(attuale.nome, attuale.descrizione, attuale.id));
                                }

                                attuale = null;
                                attuale = new Blocco();
                                // L'id del blocco è la parte di string contenuta tra le due []
                                // e senza gli spazi iniziali e finali
                                attuale.id = riga.substring(1, riga.length() - 1).trim();
//                                System.out.println("ID = " + attuale.id);
                                stato = 1;
                            }
                        }
                    } // Sono nello stato 1
                    else if (stato == 1) {
                        // La riga è lunga più di 0 caratteri
                        if (riga.length() > 0) {

                            // Divido la riga in base a =
                            String[] divi = riga.split("[=]");

                            // Tolgo gli spazi iniziali e finali
//                            for (String chi : divi) {
//                                chi = chi.trim();
//                            }

                            // Vado avanti solo se la riga divisa è composta da
                            // almeno 2 parti
                            if (divi.length == 2) {
                                if (divi[0].trim().equals("nome")) {
                                    attuale.nome = divi[1].trim();
//                                    System.out.println("NOME = " + divi[1]);
                                } else if (divi[0].trim().equals("descrizione")) {
                                    attuale.descrizione = divi[1].trim();
//                                    System.out.println("DESCR = " + divi[1]);
                                } else if (divi[0].trim().equals("uscita")) {
                                    attuale.uscite.add(divi[1].trim());
//                                    System.out.println("USCITA = " + divi[1]);
                                } else if (divi[0].trim().equals("oggetto")) {
                                    attuale.oggetti.add(divi[1].trim());
//                                    System.out.println("OBJ = " + divi[1]);
                                }
                            } // La riga non ha un = in mezzo
                            else if ((riga.startsWith("[")) && (riga.endsWith("]"))) {
                                // Ho trovato la fine di un oggetto e l'inizio di un altro
                                inserisciAttuale(attuale);
//                                locazioni.put(attuale.id, new Locazione(attuale.nome, attuale.descrizione, attuale.id));
                                attuale = null;
                                attuale = new Blocco();
                                attuale.id = riga.substring(1, riga.length() - 1).trim();
                                stato = 1;
                            }

                        }
                    }

                }

                if (attuale != null) {
                    inserisciAttuale(attuale);
                }
//                if (attuale != null) {
//                    if ((attuale.id != null) && (attuale.nome != null)) {
//                        Locazione daMettere = new Locazione(attuale.nome, attuale.descrizione, attuale.id);
//                        for (String chi : attuale.oggetti) {
//                            daMettere.aggiungiOggetto(chi);
//                        }
//
//                        for (String chi : attuale.uscite) {
//                            // Devo scansionare la stringa chi
//                            // e determinare che uscita creare
//                            String[] endo = chi.split("[ ]");
//                            if (endo.length == 4) {
//                                Uscita nuova = new Uscita(
//                                        endo[0],
//                                        new Boolean(endo[2]),
//                                        new Boolean(endo[3]),
//                                        endo[1]);
//                                daMettere.aggiungiUscita(nuova);
//                            }
//
//                        }
//
//                        locazioni.put(attuale.id, daMettere);
//                        System.out.println(daMettere.getNome());
//                        System.out.println(daMettere.getId());
//                        System.out.println(daMettere.getDescrizione());
//                    }
//                }




            } catch (IOException ex) {
//                Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
                dest.scriviDBG(ex.toString());
            } finally {
                try {
                    lettore.close();
                } catch (IOException ex) {
//                    Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
                    dest.scriviDBG(ex.toString());
                }
            }

        }

    }

    void inserisciAttuale(Blocco attuale) {
        if (attuale != null) {

            //Visualizzazione di debug
//            System.out.println("id: " + attuale.id);
//            System.out.println("nome: " + attuale.nome);
//            System.out.println("descrizione: " + attuale.descrizione);
//            for (String chi: attuale.oggetti) {
//                System.out.println("Oggetto: " + chi);
//            }
//            
//            for (String chi: attuale.uscite) {
//                System.out.println("Uscita: " + chi);
//            }


            if ((attuale.id != null) && (attuale.nome != null)) {
                Locazione daMettere = new Locazione(attuale.nome, attuale.descrizione, attuale.id);

//                System.out.println("Loc aggiunta: " + daMettere.getId());

                for (String chi : attuale.oggetti) {
                    daMettere.aggiungiOggetto(chi);
                }

//                System.out.println("Salvataggio uscite:");

                for (String chi : attuale.uscite) {
                    // Devo scansionare la stringa chi
                    // e determinare che uscita creare
                    String[] endo = chi.trim().split("[ ]");
//                    System.out.println(chi);
//                    System.out.println("endo.length = " + endo.length);
                    if (endo.length == 4) {
                        Uscita nuova = new Uscita(
                                endo[0],
                                new Boolean(endo[2]),
                                new Boolean(endo[3]),
                                endo[1]);
//                        System.out.println(nuova.getDove());
//                        System.out.println(nuova.getDirezione());
//                        System.out.println(nuova.getAperta());
//                        System.out.println(nuova.getVisibile());
                        daMettere.aggiungiUscita(nuova);
                    }

                }

                locazioni.put(attuale.id, daMettere);
//                System.out.println(daMettere.getNome());
//                System.out.println(daMettere.getId());
//                System.out.println(daMettere.getDescrizione());
            }
        }
    }

    class Ogg {

        public String nome;
        public String descrizione;
        public String posizione;
        public Boolean mobile;
    }

    class Blocco {

        String id;
        String nome;
        String descrizione;
        ArrayList<String> uscite = new ArrayList<String>();
        ArrayList<String> oggetti = new ArrayList<String>();
    }

    private void leggiBaba(File f) {
        BufferedReader lettore = null;
        try {
            lettore = new BufferedReader(new FileReader(f));
            String riga;
            Ogg o = null;


            // Stati possibili:
            // Ancora niente = 0
            // Oggetto = 1
            // NPC = 2
            // Pietra = 3
            Integer stato = 0;


            while ((riga = lettore.readLine()) != null) {
                String[] divi = riga.split("[=]");


                // Ho trovato una separazione
                if (riga.equals("----")) {
//                        stato = 0;
                    if (o != null) {

                        // Aggiungo alle mie varie liste
                        // in base al tipo di oggetto

                        // Era un Oggetto
                        if (stato == 1) {
                            oggetti.put(o.nome, new Oggetto(o.nome, o.descrizione, o.mobile));
                        }

                        // Era un NPC
                        if (stato == 2) {
                            NPC gatto = new NPC(o.nome, o.descrizione);

//                            System.out.println("Gatto - Nome: " + gatto.getNome());
//                            System.out.println("Gatto - Descrizione: " + gatto.getDescrizione());

                            if (o.posizione != null) {
                                gatto.setIdLocAttuale(o.posizione);
//                                System.out.println("Gatto - Nome: " + gatto.getIdLocAttuale());
                            }
                            listaNPC.put(o.nome, gatto);
                        }

                        if (stato == 3) {
//                            System.out.println(o.nome);
//                            System.out.println(o.descrizione);
                            pietreMiliari.put(o.nome, new PietraMiliare(o.nome, o.descrizione));
                        }

//                        System.out.println("\nL'ultimo oggetto era di tipo " + stato);
//                        System.out.println("Nome oggetto = " + o.nome);
//                        System.out.println("Descrizione oggetto = " + o.descrizione);
//                        System.out.println("Mobile oggetto = " + o.mobile);
                    }
                    stato = 0;
                    o = null;
                } else if (stato == 0) {

                    if ((divi.length == 2) && (divi[0].equals("tipo"))) {

                        // è un Oggetto
                        if (divi[1].equals("oggetto")) {
                            stato = 1;
                            o = new Ogg();
                            o.mobile = true;
                        } // è un NPC
                        else if (divi[1].equals("npc")) {
                            stato = 2;
                            o = new Ogg();
                        } // è una Pietra
                        else if (divi[1].equals("pietra")) {
                            stato = 3;
                            o = new Ogg();
                        }
                    }
                } // Ho già determinato che è un oggetto
                else if ((stato == 1) && (divi.length == 2)) {
//                    System.out.println("Oggetto: " + divi[divi.length - 1]);
                    if (divi[0].equals("nome")) {
                        o.nome = divi[1];
                    }
                    if (divi[0].equals("descrizione")) {
                        o.descrizione = divi[1];
                    }
                    if (divi[0].equals("mobile")) {
                        if (divi[1].equals("true")) {
                            o.mobile = true;
                        } else {
                            o.mobile = false;
                        }
                    }
                } // Ho già determinato che è un NPC
                else if ((stato == 2) && (divi.length == 2)) {
//                    System.out.println("NPC: " + divi[divi.length - 1]);
                    if (divi[0].equals("nome")) {
                        o.nome = divi[1];
                    }
                    if (divi[0].equals("descrizione")) {
                        o.descrizione = divi[1];
                    }
                    if (divi[0].equals("posizione")) {
                        o.posizione = divi[1];
                    }

                } // Ho già determinato che è una Pietra
                else if ((stato == 3) && (divi.length == 2)) {
//                    System.out.println("Pietra: " + divi[divi.length - 1]);
                    if (divi[0].equals("nome")) {
                        o.nome = divi[1];
                    }
                    if (divi[0].equals("descrizione")) {
                        o.descrizione = divi[1];
                    }

                }


            }
            if (o != null) {
//                System.out.println("\nL'ulitmo oggetto era di tipo " + stato);
//                System.out.println("Nome oggetto = " + o.nome);
//                System.out.println("Descrizione oggetto = " + o.descrizione);
//                System.out.println("Mobile oggetto = " + o.mobile);

            }
        } catch (IOException ex) {
//            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            dest.scriviDBG(ex.toString());
        } finally {
            try {
                lettore.close();
            } catch (IOException ex) {
//                Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
                dest.scriviDBG(ex.toString());
            }
        }

    }

    /**
     * Inizia il gioco vero e proprio. In sostanza fa partire il Thread che gestisce 
     * il mondo ed esegue la pietra miliare di nome <b>inizio</b>, se esiste.
     */
    @Override
    public void run() {
        try {

            // inizia il gioco, leggendo per prima cosa la pietra miliare
            // detta inizio, se esiste
            ArrayList l = new ArrayList();
            l.add("niente");
            interprete.callScriptFunction("inizio_func", l);


            String[] gatto = {"guarda"};
            comandoGuarda(gatto);


            tempo.start();

        // E qui entra nel ciclo che gestisce l'input, e fa finire
        // tutto quanto quando finisce il gioco
        } catch (IOException ex) {
            Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
        } catch (FSException ex) {
            Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
        }

        // E qui entra nel ciclo che gestisce l'input, e fa finire
        // tutto quanto quando finisce il gioco
        while (!finito) {
        }

    }

    /**
     * Imposta la variabile booleana finito a true, per far terminare
     * i Thread vari. Forse c'è un metodo più elegante per fare ciò,
     * devo ancora investigare bene
     */
    synchronized public void setFinito() {
        finito = true;
    }

    /**
     * Questa funzione riceve in input una stringa, e la stringa viene analizzata
     * per cercare di capire che cosa il giocatore vuole dire al Babaoglu
     * @param comando Il comando che viene passato al Babaoglu
     * @return 0 se tutto ok<br>
     * 1 se c'è stato errore di interpretazione
     */
    @Override
    public int esaminaComando(String comando) {
//        dest.scriviDBG("Gioco: comando in arrivo = " + comando);

        dest.scrivi("\n-- " + comando);
        // Tutta la logica del gioco!

        // Qui divido la stringa di comando in base allo spazio
        // Non è una buonissima idea, però per ora va bene
        // Il problema è che non gestisce gli oggetti con nome composto...
        // Va beh si arrangiano
        String[] divi = comando.split(" ");

        // Proseguo solo se c'è almeno una stringa in divi, altrimenti
        // non c'è nemmeno un comando!
        if (divi.length >= 1) {

            // Ora decido che cosa fare a seconda del comando
            // NOTA: il comando lo rendo in minuscolo
            String c = divi[0].toLowerCase();


            if (c.equals("usa")) {
                comandoUsa(divi);
            } else if (c.equals("vai")) {
                comandoVai(divi);
            } else if (c.equals("apri")) {
                comandoApri(divi);
            } else if (c.equals("lascia")) {
                comandoLascia(divi);
            } else if (c.equals("prendi")) {
                comandoPrendi(divi);
            } else if (c.equals("inventario") || c.equals("inv")) {
                comandoInv();
            } else if (c.equals("guarda")) {
                comandoGuarda(divi);
            } else if (c.equals("uscite")) {
                comandoUscite(divi);
            } else if (c.equals("parla")) {
                comandoParla(divi);
            }



        }


        return 1;
    }

    private void comandoUsa(String[] divi) {
        // Non ci sono parametri:
        if (divi.length == 1) {
            dest.scrivi("Usa che cosa?");
        }

        // C'è 1 parametro: uso quell'oggetto
        if (divi.length >= 2) {
            String o = divi[1].toLowerCase();

            // Vedo se è contenuto negli oggetti
            if (giocatore.haOggetto(o)) {
                try {
                    ArrayList param = new ArrayList();
                    param.add("usa");
                    
                    if (divi.length == 3) {
                        param.add(divi[2]);
                    } else {
                        param.add("");
                    }
                    
                    interprete.callScriptFunction(o + "_func", param);
                    
                } catch (IOException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                } catch (FSException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                }
            } else if (locazioni.get(giocatore.getLocazione()).haOggetto(o)) {
                try {
                    ArrayList param = new ArrayList();
                    param.add("usa");
                    param.add("");
                    interprete.callScriptFunction(o + "_func", param);
                } catch (IOException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                } catch (FSException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                }
            } else {
                dest.scrivi("Non hai " + o + "!");
            }
        } else if (divi.length == 3) {
            Boolean oggPassivo = false;
            Boolean oggAttivo = false;

            if (giocatore.haOggetto(divi[1].toLowerCase())) {
                oggAttivo = true;
            }
            if (giocatore.haOggetto(divi[2].toLowerCase())) {
                oggPassivo = true;
            } else if (locazioni.get(giocatore.getLocazione()).haOggetto(divi[2].toLowerCase())) {
                oggAttivo = true;
            }

            if (oggAttivo && oggPassivo) {
                try {
                    ArrayList param = new ArrayList();
                    param.add(divi[1]);
                    interprete.callScriptFunction(divi[2].toLowerCase() + "_func", param);
                } catch (IOException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                } catch (FSException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                }


            } else if (!oggAttivo) {
                dest.scrivi("Non hai " + divi[1].toLowerCase());
            } else if (!oggPassivo) {
                dest.scrivi(divi[2].toLowerCase() + " non disponibile!");
            }
        }
    }

    private void comandoParla(String[] divi) {
        if (divi.length >= 2) {
            String who = divi[1];

            if (this.getListaNPC().contains(who)) {
                if (giocatore.getLocazione().equals(listaNPC.get(who).getIdLocAttuale())) {
                    try {
                        String nomeFunzione = who + "_func";
                        ArrayList<String> param = new ArrayList<String>();
                        param.add("parla");
                        
                        if (divi.length > 2) {
                            for (int i = 2; i < divi.length; i++) {
                                param.add(divi[i]);
                            }
                        }
                        else param.add("");

                        interprete.callScriptFunction(nomeFunzione, param);
                    } catch (IOException ex) {
//                        Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (FSException ex) {
//                        Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else {
                    dest.scrivi("\n Non posso parlare con " + who);
                }
            } else {
                dest.scrivi("\n Non posso parlare con " + who);
            }


        } else {
            dest.scrivi("\nParla con chi?");
        }


    }

    private void comandoVai(String[] divi) {
        if (divi.length == 2) {
            // Ora vedo se può andare dove vuole andare
            Locazione l = locazioni.get(giocatore.getLocazione());

            locazioni.get(giocatore.getLocazione()).setVisibile(divi[1].toLowerCase().trim());

            //L'uscita è presente?
            if (l.haUscita(divi[1].toLowerCase())) {

//                System.out.println("Uscita " + divi[1]  + " presente.");
                // Recupero il numero dell'uscita
                Integer i = l.getNumeroUscita(divi[1].toLowerCase());



                // Se l'uscita i è aperta, ci vado!
                if ((i != -1) && l.getAperta(i)) {

//                    System.out.println("Uscita " + i + " aperta.");
                    giocatore.setLocazione(l.dovePorta(i));

                    // Eseguo la funzione della locazione attuale
                    ArrayList param = new ArrayList();
                    param.add("normale"); // oppure niente, quello che è

                    try {
                        interprete.callScriptFunction(giocatore.getLocazione() + "_func", param);
                    } catch (IOException ex) {
                        Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (FSException ex) {
                        Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                    }

                    // Scrivo la descrizione della locazione attuale
//                    System.out.println("Locazione attuale = " + giocatore.getLocazione());
//                    Locazione gatto = locazioni.get(giocatore.getLocazione());
//                    System.out.println(gatto.getDescrizione());

//                    dest.scrivi(cane);
//                    esaminaComando("guarda");
                    String[] imbranato = {"guarda"};
                    comandoGuarda(imbranato);
//                    dest.scrivi(
//                            locazioni.get
//                            (giocatore.getLocazione())
//                            .getDescrizione());
                } else {
                    dest.scrivi("Tu... non puoi... PASSARE!");
                }
            } else {
                dest.scrivi("Non esiste l'uscita " + divi[1].toLowerCase());
            }
        } else {
            dest.scrivi("Vai dove?");
        }
    }

    private void comandoApri(String[] divi) {
        if (divi.length == 2) { // Apro un'uscita direttamente
            String uscita = divi[1].toLowerCase();

            // Controllo se la locazione in cui si trova il giocatore ha
            // l'uscita indicata
            if (locazioni.get(giocatore.getLocazione()).haUscita(uscita)) {
                try {

                    locazioni.get(giocatore.getLocazione()).setVisibile(uscita);

//                    System.out.println("L'uscita '" + uscita + "' esiste in " + giocatore.getLocazione());

                    // Chiamo la funzione dell'uscita relativa
                    String nomeFunzione = giocatore.getLocazione() + "_" + uscita + "_func";
                    ArrayList param = new ArrayList();
                    param.add("apri");
                    interprete.callScriptFunction(nomeFunzione, param);
                } catch (IOException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                } catch (FSException ex) {
                    Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                }
            } else {
                dest.scrivi("L'uscita " + uscita + " non esiste.");
            }
        } else if (divi.length == 3) { // Apro un'uscita con un oggetto
            String uscita = divi[1].toLowerCase();
            String ogge = divi[2].toLowerCase();

            // Controllo se la locazione in cui si trova il giocatore ha
            // l'uscita indicata
            if (locazioni.get(giocatore.getLocazione()).haUscita(uscita)) {
                if (giocatore.haOggetto(ogge)) {
                    try {
                        // Chiamo la funzione dell'uscita relativa
                        String nomeFunzione = giocatore.getLocazione() + "_" + uscita + "_func";
                        ArrayList param = new ArrayList();
                        param.add("apri");
                        param.add(ogge);
                        interprete.callScriptFunction(nomeFunzione, param);
                    } catch (IOException ex) {
                        Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                    } catch (FSException ex) {
                        Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else {
                    dest.scrivi("Non hai " + ogge);
                }
            } else {
                dest.scrivi("L'uscita " + uscita + " non esiste.");
            }
        } else {
            dest.scrivi("Che cosa apri");
        }
    }

    private void comandoLascia(String[] divi) {
        if (divi.length >= 2) {
            for (int c = 1; c < divi.length; c++) {
                String ogge = divi[c];
                if (giocatore.haOggetto(ogge)) {
                    giocatore.togliOggetto(ogge);
                    locazioni.get(giocatore.getLocazione()).aggiungiOggetto(ogge);

                    dest.scrivi("\nHai lasciato: " + ogge);
                }

            }
        } else {
            dest.scrivi("Che cosa lasci?");
        }
    }

    private void comandoPrendi(String[] divi) {
        if (divi.length >= 2) {
            String locAtt = giocatore.getLocazione();
            for (int c = 1; c < divi.length; c++) {

                if (locazioni.get(locAtt).haOggetto(divi[c])) {

                    if (oggetti.get(divi[c]).getMobile()) {

                        giocatore.aggiungiOggetto(divi[c]);
                        locazioni.get(locAtt).raccogliOggetto(divi[c]);
                        dest.scrivi("\nHai preso: " + divi[c]);
                    } else {
                        dest.scrivi("\nNon puoi prendere: " + divi[c]);
                    }



                } else {
                    dest.scrivi("Non c'è " + divi[c] + ", qui.");
                }
            }
        } else {
            dest.scrivi("Che cosa prendi?");
        }
    }

    private void comandoInv() {
        String[] inv = giocatore.getInventario();

        if (inv.length == 0) {
            dest.scrivi("\nNon hai niente con te.");
        } else {
            dest.scrivi("\nInventario:");

            for (String chi : inv) {
                dest.scrivi("* " + chi);
            }

        }

    }

    private void comandoGuarda(String[] divi) {
        if (divi.length >= 2) {
            for (int c = 1; c < divi.length; c++) {
                // Comincio a controllare se il player ha quell'oggetto
                if (giocatore.haOggetto(divi[c])) {
                    dest.scrivi("\n" + divi[c] + ": " + oggetti.get(divi[c]).getDescrizione());
                } // Controllo se c'è un oggetto di quel nome nella locazione
                else if (locazioni.get(giocatore.getLocazione()).haOggetto(divi[c])) {
                    dest.scrivi("\n" + divi[c] + ": " + oggetti.get(divi[c]).getDescrizione());
                } // Controllo se c'è un player di quel nome in questa locazione
                else if (listaNPC.containsKey(divi[c])) {
                    String doveStaLNPC = listaNPC.get(divi[c]).getIdLocAttuale();

//                    System.out.println("Locazione di "  + divi[c] + " = " + doveStaLNPC);

                    if (doveStaLNPC.equals(giocatore.getLocazione())) {

//                        System.out.println("Locazione corrispondente!");

//                        String uku = listaNPC.get(divi[c]).getDescrizione();
//                        System.out.println("Uku: '" + uku + "'");

                        dest.scrivi("\nGuardi " + divi[c] + ": " + listaNPC.get(divi[c]).getDescrizione());
                    } else {
                        dest.scrivi(divi[c] + " non è qui.");
                    }
                } // Non so più che cosa controllare
                else {
                    dest.scrivi("Non so che cosa guardare...");
                }
            }
        } else if (divi.length == 1) {
//            Locazione gatto = locazioni.get(giocatore.getLocazione());
//            gatto.getId();
//            System.out.println(gatto.getDescrizione());
//            System.out.println("Locaz. attuale = " + giocatore.getLocazione());

            dest.scrivi("\n" + ((Locazione) locazioni.get(giocatore.getLocazione())).getNome());
            dest.scrivi(((Locazione) locazioni.get(giocatore.getLocazione())).getDescrizione());
            String[] listaOggetti = locazioni.get(giocatore.getLocazione()).getListaOggetti();

            if (listaOggetti.length > 0) {
                dest.scrivi("Oggetti:");

                for (int i = 0; i < listaOggetti.length; i++) {
                    dest.scrivi("* " + listaOggetti[i]);
                }

            }

            ArrayList<String> listaUscite = getUsciteVisibili(giocatore.getLocazione());

            if (listaUscite.size() > 0) {
                dest.scrivi("Uscite: ");

                for (String chi : listaUscite) {
                    dest.scrivi("* " + chi);
                }
            }

            Set<String> listaPeople = listaNPC.keySet();
            ArrayList<String> persone = new ArrayList<String>();

            for (String who : listaPeople) {
                if (listaNPC.get(who).getIdLocAttuale().equals(giocatore.getLocazione())) {
                    persone.add(who);
                }
            }

            if (persone.size() > 0) {
                dest.scrivi("Persone:");
                for (String who : persone) {
                    dest.scrivi("* " + who);
                }
            }



        }
    }

    private void comandoUscite(String[] divi) {
//        System.out.println("USCITE AFSADAS");

        Locazione now = locazioni.get(giocatore.getLocazione());
        ArrayList<String> direzioneUsciteVisibili = now.getDirezioneUsciteVisibili();

//        System.out.println(direzioneUsciteVisibili.size());
        dest.scrivi("\nUscite:");
        for (String gatto : direzioneUsciteVisibili) {
            dest.scrivi(" * " + gatto);
        }

    }

    /**
     * Ritorna la lista delle locazioni, per id
     * @return Un Set contenente la lista delle locazioni.
     */
    public Set<String> getListaLocazioni() {
        return locazioni.keySet();
    }

    /**
     * Ritorna il nome <i>esteso</i> di una locazione
     * @param dove L'idLoc della locazione di cui si vuole il nome
     * @return Il nome esteso di una locazione
     */
    public String getNomeEstesoLocazione(String dove) {
        return locazioni.get(dove).getNome();
    }

    /**
     * Ritorna la lista degli NPC
     * @return Un Set contenente tutti gli NPC
     */
    public Set<String> getListaNPC() {
        return listaNPC.keySet();
    }

    /**
     * Ritorna la descrizione di un NPC scelto per nome
     * @param nome Il nome dell'NPC da interpellare
     * @return La sua descrizione
     */
    public String getDescrizioneNPC(String nome) {
        if (listaNPC.containsKey(nome)) {
            return listaNPC.get(nome).getDescrizione();
        } else {
            return null;
        }
    }

    /**
     * Ritorna la posizione attuale di un NPC
     * @param nome Il nome dell'NPC da verificare
     * @return La posizione in cui esso si trova
     */
    public String getLocazioneNPC(String nome) {
        if (listaNPC.containsKey(nome)) {
            return listaNPC.get(nome).getIdLocAttuale();
        } else {
            return null;
        }
    }

    /**
     * Ritorna la descrizione di un oggetto, scelto per nome
     * @param nome Il nome dell'oggetto da interpellare
     * @return La sua descrizione
     */
    public String getDescrizioneOggetto(String nome) {
        if (oggetti.containsKey(nome)) {
            return oggetti.get(nome).getDescrizione();
        } else {
            return null;
        }
    }

    /**
     * Ritorna il numero di uscite di una determinata locazione
     * @param dove L'idLoc della locazione da interpelllare
     * @return Il numero di uscite che ha
     */
    public Integer getNumeroUscite(String dove) {
        return locazioni.get(dove).getNumeroUscite();
    }

    /**
     * Ritorna un ArrayList contenente le il Dove e la Direzione di
     * ogni uscita della locazione indicata.<br>
     * Il Dove e la Direzione sono separate da un "|", così da poter essere
     * scansionate.<br>
     * Questa funzione esiste praticamente per scrivere le frasi come "L'NPC esce a nord".
     * @param dove
     * @return
     */
    public ArrayList<String> getUsciteEDirezione(String dove) {
        if (locazioni.containsKey(dove)) {
            return locazioni.get(dove).getDirezioneEDoveUscite();
        } else {
            return null;
        }

    }

    /**
     * Ritorna la lista delle uscite <b>visibili</b> di una certa locazione
     * @param dove La locazione (per idLoc) da controllare
     * @return La lista delle uscite visibili
     */
    public ArrayList<String> getUsciteVisibili(String dove) {
        ArrayList<String> risultato = null;
        if (locazioni.containsKey(dove)) {
            risultato = locazioni.get(dove).getDirezioneUsciteVisibili();
        }

        return risultato;
    }

    /**
     * Ritorna lo stato di un'uscita di una determinata locazione
     * @param dove La locazione da intervistare
     * @param quale il numero dell'uscita da controllare
     * @return <b>true</b> se l'uscita è aperta<br><b>false</b> se l'uscita è chiusa
     */
    public Boolean uscitaAperta(String dove, Integer quale) {
        return locazioni.get(dove).getAperta(quale);
    }

    /**
     * Ritorna l'idLoc della locazione a cui punta una determinata uscita
     * @param dove La locazione di cui controllare l'uscita
     * @param quale L'uscita da controllare
     * @return L'idLoc della locazione destinazione di quell'uscita
     */
    public String direzioneUscita(String dove, Integer quale) {
        return locazioni.get(dove).dovePorta(quale);
    }

    /**
     * Controlla se il giocatore ha un oggetto oppure no
     * @param nome Il nome dell'oggetto
     * @return <b>true</b> se ce l'ha, <b>false</b> se non ce l'ha
     */
    public Boolean playerHaOggetto(String nome) {
        return giocatore.haOggetto(nome);

    }

    /**
     * Controlla se il giocatore ha una Pietra oppure no
     * @param nome Il nome della Pietra
     * @return <b>true</b> se ce l'ha, <b>false</b> se non ce l'ha
     */
    public Boolean playerHaPietra(String nome) {
        return giocatore.haPietra(nome);
    }

    /**
     * Ritorna la posizione attuale di un dato NPC
     * @param nome Il nome dell'NPC da controllare
     * @return Il nome dell'idLoc dove si trova
     */
    public String posizioneNPC(String nome) {
        return listaNPC.get(nome).getIdLocAttuale();
    }

    /**
     * Ritorna la posizione attuale del Giocatore
     * @return L'idLoc del giocatore
     */
    public String posizionePlayer() {
        return giocatore.getLocazione();
    }

    /**
     * Da' un oggetto al giocatore
     * @param nome Il nome dell'oggetto da dare
     */
    public void daiOggettoPlayer(String nome) {
        if (oggetti.containsKey(nome)) {
            giocatore.aggiungiOggetto(nome);
            scrivi("-- Ora hai: " + nome);
        }
    }

    /**
     * Toglie un oggetto al giocatore
     * @param nome Il nome del giocatore
     */
    public void togliOggettoPlayer(String nome) {
        giocatore.togliOggetto(nome);
    }

    /**
     * Pone un oggetto in una locazione
     * @param nome Il nome dell'oggetto
     * @param loc Il nome della locazione
     */
    public void daiOggettoLoc(String nome, String loc) {
        if (locazioni.containsKey(loc) && oggetti.containsKey(nome)) {
            locazioni.get(loc).aggiungiOggetto(nome);
        }
    }

    /**
     * Toglie un oggetto da una locazione
     * @param nome Il nome dell'oggetto
     * @param loc Il nome della locazione
     */
    public void togliOggettoLoc(String nome, String loc) {
        if (locazioni.containsKey(loc) && oggetti.containsKey(nome)) {
            locazioni.get(loc).raccogliOggetto(nome);
        }
    }

    /**
     * Da' una Pietra Miliare al giocatore
     * @param nome Il nome della Pietra Miliare da aggiungere
     */
    public void daiPietra(String nome) {
        if (pietreMiliari.containsKey(nome)) {
            try {
                giocatore.aggiungiPietra(pietreMiliari.get(nome).getNome());
                ArrayList param = new ArrayList();
                param.add("dai");
                interprete.callScriptFunction(nome + "_func", param);
            } catch (IOException ex) {
                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
            } catch (FSException ex) {
                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    /**
     * Toglie una Pietra Miliare al giocatore
     * @param nome Il nome della Pietra Miliare da levare
     * @return Il risultato dell'operazione
     */
    public Boolean togliPietra(String nome) {

        if (giocatore.haPietra(nome)) {
            try {
                ArrayList param = new ArrayList();
                param.add("togli");
                interprete.callScriptFunction(nome + "_func", param);
            } catch (IOException ex) {
                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
            } catch (FSException ex) {
                Logger.getLogger(Gioco.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return giocatore.togliPietra(nome);
    }

    /**
     * Scrive qualcosa a schermo
     * @param cosa La String da scrivere
     */
    public void scrivi(String cosa) {
        dest.scrivi(cosa);
    }

    public void scriviDBG(String cosa) {
        dest.scriviDBG(cosa);
    }

    /**
     * Chiede qualcosa al giocatore. La domanda viene presentata come una lista <br>
     * di righe numerate, e il giocatore dovrà ritornare il numero corrispondente ad una delle uscite.<br>
     * L'implementazione di tutto ciò spetta al <b>BabaIO</b>.
     * @param testo Il testo della domanda
     * @param domande Le varie opzioni poste al giocatore
     * @return La risposta del giocatore.
     */
    public Integer domanda(String testo, ArrayList<String> domande) {
        return dest.domanda(testo, domande);
    }

    /**
     * Sposta un NPC in un'altra locazione
     * @param chi L'NPC da spostare
     * @param dove La locazione in cui spostare
     */
    public void spostaNPC(String chi, String dove) {
        if (listaNPC.containsKey(chi) && locazioni.containsKey(dove)) {
            listaNPC.get(chi).setIdLocAttuale(dove);
        }
    }

    /**
     * Sposta il giocatore in un'altra locazione
     * @param dove La locazione dove spostare il giocatore
     */
    public void spostaPlayer(String dove) {
        if (locazioni.containsKey(dove)) {
            giocatore.setLocazione(dove);
        }
    }

    /**
     * Svuota l'inventario del giocatore
     */
    public void svuotaInventario() {
        giocatore.svuotaInventario();
    }

    /**
     * Apre l'uscita di una locazione
     * @param dove La locazione
     * @param quale L'uscita da aprire
     */
    public void apriUscita(String dove, String direzione) {
        if (locazioni.containsKey(dove)) {
            try {
                locazioni.get(dove).setAperta(direzione);
            } catch (Exception ex) {
//                System.out.println("Uscita non valida");
            }
        }
    }

    /**
     * Chiude l'uscita di una locazione
     * @param dove La locazione
     * @param quale L'uscita da chiudere
     */
    public void chiudiUscita(String dove, String direzione) {
        if (locazioni.containsKey(dove)) {
            try {
                locazioni.get(dove).setChiusa(direzione);
            } catch (Exception ex) {
//                System.out.println("Uscita non valida");
            }
        }
    }

    /**
     * Rende visibile l'uscita di una locazione
     * @param dove La locazione
     * @param quale L'uscita da rendere visibile
     */
    public void rendiVisibile(String dove, String direzione) {
        if (locazioni.containsKey(dove)) {
            try {
                locazioni.get(dove).setVisibile(direzione);
            } catch (Exception ex) {
            // niente
            }

        }
    }

    /**
     * Rende invisibile una certa uscita di una locazione
     * @param dove La locazione la cui uscita voglio rendere invisibile
     * @param quale L'uscita da rendere invisibile
     */
    public void rendiInvisibile(String dove, String direzione) {
        if (locazioni.containsKey(dove)) {
            try {
                locazioni.get(dove).setVisibile(direzione);
            } catch (Exception ex) {
            // Niente
            }

        }
    }
    // OCCHIO: le locazioni sono memorizzate per ID, non per nome!
    LinkedHashMap<String, Locazione> locazioni;
    LinkedHashMap<String, Oggetto> oggetti;
    LinkedHashMap<String, PietraMiliare> pietreMiliari;
    LinkedHashMap<String, NPC> listaNPC;
    Player giocatore;
//    private Tempo tempo;
    Thread tempo;
    BabaIO dest;
    Boolean finito;
    FScript interprete;
}
