/*
 * Locazione.java
 *
 * Created on 28 novembre 2007, 17.40
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package babaoglu;

import java.util.*;

/**
 * La locazione � uno degli oggetti fondamentali del Babaoglu Engine.
 * @author dario
 */
public class Locazione {

    /**
     * Crea una nuova locazione
     * @param n il nome della locazione
     * @param d la descrizione della locazione
     * @param i l'id della locazione
     */
    public Locazione(String n, String d, String i) {
        nome = n;
        descrizione = d;
        id = i;

        listaUscite = new ArrayList<Uscita>();
        listaOggetti = new ArrayList<String>();
    }

    /**
     * Aggiunge un'uscita alla locazione corrente
     * @param u L'uscita da aggiungere
     * @return il risultato dell'aggiunta dell'uscita alla lista delle uscite
     */
    public Boolean aggiungiUscita(Uscita u) {
        return listaUscite.add(u);
    }

    /**
     * Aggiunge un oggetto alla lista degli oggetti mantenuti nella locazione
     * @param o Il nome dell'oggetto da aggiungere
     * @return Il risultato dell'aggiunta dell'oggetto alla lista
     */
    public Boolean aggiungiOggetto(String o) {
        return listaOggetti.add(o);
    }

    public Boolean haOggetto(String o) {

//        System.out.println("Oggetto richiesto = '" + o + "'");

        Boolean ris = false;
        Integer c = 0;
        while ((c < listaOggetti.size()) && (!ris)) {

//            System.out.println("Esamino l'oggetto '" + listaOggetti.get(c) + "'");

            if (listaOggetti.get(c).equals(o)) {
                ris = true;
            }
            c++;
        }
        return ris;
    }

    /**
     * Restituisce il nome della locazione
     * @return Il nome della locazione
     */
    public String getNome() {
        return nome;
    }

    /**
     * Restituisce la descrizione della locazione
     * @return La descrizione della locazione
     */
    public String getDescrizione() {
        return descrizione;
    }

    /**
     * Restituisce l'id della locazione
     * @return L'id della locazione
     */
    public String getId() {
        return id;
    }

    /**
     * Ritorna il <I>numero</I> di uscite della locazione attuale,
     * siano esse visibili, invisibili, aperte o chiuse.
     * @return Il numero di uscite della locazione
     */
    public int getNumeroUscite() {
        return listaUscite.size();
    }

    /**
     * Ritorna la lista delle uscite visibile di questa locazione
     * @return La lista delle uscite visibili
     */
    public ArrayList<String> getDirezioneUsciteVisibili() {
        ArrayList<String> risultato = new ArrayList<String>();
//        System.out.println("Numero uscite di " + this.nome + " = " + listaUscite.size());

        for (int c = 0; c < listaUscite.size(); c++) {
//            System.out.println(this.nome + ": " + listaUscite.get(c).getDirezione());
            if (listaUscite.get(c).getVisibile()) {
                risultato.add(listaUscite.get(c).getDirezione());
            }
        }

        return risultato;
    }

    /**
     * Ritorna una lista contenente le Direzioni e il Dove di tutte le uscite
     * comprese quelle non visibili.<br>Sono separate da una pipe "|"
     * @return La lista delle uscite, visbili e invisibili, separate da una "|"
     */
    public ArrayList<String> getDirezioneEDoveUscite() {
        ArrayList<String> ris = new ArrayList<String>();

        for (Uscita c : listaUscite) {
            ris.add(c.getDirezione() + "|" + c.getDove());

        }

        return ris;
    }

    /**
     * Verifica la presenza di un'uscita con quella Direzione.
     * @param d La Direzione da verificare
     * @return True se l'uscita esiste, False se no
     */
    public Boolean haUscita(String d) {
        Boolean ris = false;
        Integer c = 0;
        while ((!ris) && (c < listaUscite.size())) {
            if (listaUscite.get(c).getDirezione().equals(d)) {
                ris = true;
            }
            c++;
        }
        return ris;
    }

    /**
     * Restituisce l'id della locazione cui l'uscita indicata porta.
     * Il numero di uscita � il numero di posizione nell'ArrayList delle uscite,
     * e corrisponde con quello ritornato da getNumeroUscite()
     * @param u Il numero di uscita  da controllare
     * @return l'idLoc della locazione cui porta l'uscita<BR>
     * <B>null</B> in caso di errore
     */
    public String getIdUscita(int u) {
        if (u < listaUscite.size()) {
            return listaUscite.get(u).getDove();
        } else {
            return null;
        }
    }

    /**
     * Ritorna il numero dell'uscita a partire dal Dove
     * @param d Il Dove da verificare
     * @return Il numero dell'uscita, se esiste. -1 se non c'�
     */
    public Integer getNumeroUscita(String d) {


        // Controllo
//        System.out.println("Direzione richiesta = '" + d + "'");
//        for (int i = 0; i < listaUscite.size(); i++) {
//            System.out.println(listaUscite.get(i).getDirezione());
//        }


        Boolean presente = false;
        Integer quale = 0;
        while ((!presente) && (quale < listaUscite.size())) {
            if (listaUscite.get(quale).getDirezione().equals(d)) {
                presente = true;
            }
            quale++;
        }

        if (presente) {
            return (quale - 1);
        } else {
            return -1;
        }
    }

    public Boolean getAperta(int u) {

//        System.out.println("Uscita richiesta = " + u);

        //Controllo
//        for (int i = 0; i < listaUscite.size(); i ++) {
//            System.out.print("listaUscite(" + i +") " + listaUscite.get(i).getDirezione());
//            System.out.print(" " + listaUscite.get(i).getDove());
//            System.out.print(" " + listaUscite.get(i).getAperta());
//            System.out.println(" " + listaUscite.get(i).getVisibile());
//        }
//        
//        System.out.println("Fine lista");

        if ((u < listaUscite.size()) && (u >= 0)) {
            return listaUscite.get(u).getAperta();
        } else {
            return false;
        }
    }

    public void setAperta(int u) {

//        System.out.println("Uscita richiesta = " + u);
//        System.out.println("Numero uscite = "  + listaUscite.size());

        if (u < listaUscite.size()) {
            listaUscite.get(u).setAperta(true);
        }
    }

    public void setAperta(String direzione) {
        try {
            listaUscite.get(this.getNumeroUscita(direzione)).setAperta(true);
        } catch (Exception ex) {
//            System.out.println("Uscita non valida");
        }

    }

    public void setChiusa(int u) {
        if (u < listaUscite.size()) {
            listaUscite.get(u).setAperta(false);
        }
    }
    
    public void setChiusa(String direzione) {
                try {
            listaUscite.get(this.getNumeroUscita(direzione)).setAperta(false);
        } catch (Exception ex) {
//            System.out.println("Uscita non valida");
        }
    }

    public void setVisibile(int u) {
        if (u < listaUscite.size()) {
            listaUscite.get(u).setVisibile(true);
        }
    }

    public void setVisibile(String direzione) {

        try {
            listaUscite.get(this.getNumeroUscita(direzione)).setVisibile(true);
        } catch (Exception ex) {
//            System.out.println("Uscita non valida");
        }

    }

    public void setInvisibile(int u) {
        if (u < listaUscite.size()) {
            listaUscite.get(u).setVisibile(false);
        }
    }

    public void setInvisibile(String direzione) {
        try {
            listaUscite.get(this.getNumeroUscita(direzione)).setVisibile(false);
        } catch (Exception ex) {
//            System.out.println("Uscita non valida");
        }
    }

    public String dovePorta(int u) {
        if (u < listaUscite.size()) {
//            return listaUscite.get(u).getDirezione();
            return listaUscite.get(u).getDove();
        } else {
            return "";
        }
    }

    /**
     * Restituisce una lista contentente i <I>nomi</I> delle uscite visibili nella
     * locazione corrente.
     * @return La lista dei nomi delle uscite visibili
     */
//    public ArrayList<String> getDirezioneUsciteVisibili() {
//        ArrayList<String> risultato = new ArrayList<String>();
//
//        for (int c = 0; c < listaUscite.size(); c++) {
//            if (listaUscite.get(c).getVisibile()) {
//                risultato.add(listaUscite.get(c).getDirezione());
//            }
//        }
//
//        return risultato;
//    }
    /**
     * Questa funzione raccoglie l'oggetto col nome specificato come parametro.<BR>
     * Se l'oggetto esiste, esso viene ritornato, altrimenti ritorna <B>null</b>.
     * <BR><BR>
     * Da notare che l'oggetto raccolto viene <I>cancellato</i> dalla lista degli
     * oggetti presenti nella locazione!
     * @param nome Il nome dell'oggetto da cancellare
     * @return Il nome dell'Oggetto, se trovato, altrimenti <b>null</b>
     */
    public String raccogliOggetto(String nome) {
        int c = 0;
        Boolean trovato = false;
        String risultato = null;

        while ((c < listaOggetti.size()) && !trovato) {
            if (listaOggetti.get(c).equals(nome)) {
                trovato = true;
            }
            if (!trovato) {
                c++;
            }
        }

        if (trovato) {
            risultato = listaOggetti.get(c);
            listaOggetti.remove(c);
        }

        return risultato;
    }

    public String[] getListaOggetti() {
        String[] a = new String[listaOggetti.size()];
        a = listaOggetti.toArray(a);
        return a;
    }
    private String nome;
    private String descrizione;
    private String id;
    private ArrayList<Uscita> listaUscite;
    private ArrayList<String> listaOggetti;
}
