/*
 * Oggetto.java
 *
 * Created on 28 novembre 2007, 17.48
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package babaoglu;

/**
 * La classe <B>Oggetto</B> descrive un oggetto del Babaoglu Engine.<BR><BR>
 * Un oggetto è caratterizzato da:
 * <UL>
 *    <LI><B>Nome</B>: il nome dell'oggetto, che deve essere univoco
 *    <LI><B>Descrizione</B>: la descrizione dell'oggetto
 *    <LI><B>Mobile</B>: una variabile di stato che stabilisce se un oggetto è portatile o no.
 * </UL>
 * Gli oggetti non portatili non possono essere raccolti da un giocatore. 
 * Da notare che un oggetto <I>non può</I> cambiare status di mobilità durante il gioco.
 * <BR><BR>
 * Ad ogni oggetto è associata una funzione di <B>FScript</B>, con un nome standard:
 * <B>nomeoggetto_func(String arg)</B>, e serve per compiere azioni su quell'oggetto.
 * <BR><BR>
 * La presenza della funzione FScript non è obbligatoria.
 * @author dario
 */
public class Oggetto {
    
    /**
     * Creates a new instance of Oggetto
     * @param n Il nome dell'oggetto
     * @param d La descrizione dell'oggetto
     * @param m La mobilità dell'oggetto:<BR>
     * TRUE = è mobile<BR>
     * FALSE = non è mobile
     */
    public Oggetto(String n, String d, Boolean m) {
        nome = n;
        descrizione = d;
        mobile = m;
    }
    
    /**
     * Ritorna il nome dell'Oggetto
     * @return il nome dell'Oggetto
     */
    public String getNome() {
        return nome;
    }
    
    /**
     * Ritorna la descrizione dell'oggetto
     * @return La descrizione dell'oggetto
     */
    public String getDescrizione() {
        return descrizione;
    }
    
    /**
     * Ritorna lo stato di mobilità di un oggetto.
     * @return TRUE = l'oggetto è mobile<BR>
     * FALSE = l'oggetto non è mobile
     */
    public Boolean getMobile() {
        return mobile;
    }
    
    private String nome;
    private String descrizione;
    private volatile Boolean mobile;
    
}
